@@ -14,7 +14,10 @@ from distributed.utils_test import slowinc
 from tornado.ioloop import IOLoop
 from tornado.platform.asyncio import BaseAsyncIOLoop
 
-from distributed.asyncio import AioClient, AioFuture, as_completed, wait
+from distributed.client import Future
+from distributed.variable import Variable
+from distributed.asyncio import AioClient
+from distributed.asyncio import as_completed, wait
 from distributed.utils_test import inc, div
 
 
@@ -46,18 +49,17 @@ async def test_coro_test():
 
 @coro_test
 async def test_asyncio_start_close():
-    c = await AioClient(processes=False)
+    async with AioClient(processes=False) as c:
+        assert c.status == 'running'
+        # AioClient has installed its AioLoop shim.
+        assert isinstance(IOLoop.current(instance=False), BaseAsyncIOLoop)
 
-    assert c.status == 'running'
-    # AioClient has installed its AioLoop shim.
-    assert isinstance(IOLoop.current(instance=False), BaseAsyncIOLoop)
+        result = await c.submit(inc, 10)
+        assert result == 11
 
-    result = await c.submit(inc, 10)
-    assert result == 11
-
-    await c.close()
-    assert c.status == 'closed'
-    assert IOLoop.current(instance=False) is None
+        await c.close()
+        assert c.status == 'closed'
+        # assert IOLoop.current(instance=False) is None
 
 
 @coro_test
@@ -66,7 +68,7 @@ async def test_asyncio_submit():
         x = c.submit(inc, 10)
         assert not x.done()
 
-        assert isinstance(x, AioFuture)
+        assert isinstance(x, Future)
         assert x.client is c
 
         result = await x.result()
@@ -86,7 +88,7 @@ async def test_asyncio_future_await():
         x = c.submit(inc, 10)
         assert not x.done()
 
-        assert isinstance(x, AioFuture)
+        assert isinstance(x, Future)
         assert x.client is c
 
         result = await x
@@ -106,7 +108,7 @@ async def test_asyncio_map():
         L1 = c.map(inc, range(5))
         assert len(L1) == 5
         assert isdistinct(x.key for x in L1)
-        assert all(isinstance(x, AioFuture) for x in L1)
+        assert all(isinstance(x, Future) for x in L1)
 
         result = await L1[0]
         assert result == inc(0)
@@ -167,7 +169,8 @@ async def test_asyncio_get():
         result = await c.get({}, [])
         assert result == []
 
-        result = await c.get({('x', 1): (inc, 1), ('x', 2): (inc, ('x', 1))},
+        result = await c.get({('x', 1): (inc, 1),
+                              ('x', 2): (inc, ('x', 1))},
                              ('x', 2))
         assert result == 3
 
@@ -313,25 +316,51 @@ async def test_asyncio_run_coroutine():
 @slow
 @coro_test
 async def test_asyncio_restart():
-    c = await AioClient(processes=False)
+    async with AioClient(processes=False) as c:
+        assert c.status == 'running'
+        x = c.submit(inc, 1)
+        assert x.key in c.refcount
 
-    assert c.status == 'running'
-    x = c.submit(inc, 1)
-    assert x.key in c.refcount
+        await c.restart()
+        assert x.key not in c.refcount
 
-    await c.restart()
-    assert x.key not in c.refcount
+        key = x.key
+        del x
+        import gc
+        gc.collect()
 
-    key = x.key
-    del x
-    import gc
-    gc.collect()
-
-    assert key not in c.refcount
-    await c.shutdown()
+        assert key not in c.refcount
 
 
 @coro_test
 async def test_asyncio_nanny_workers():
     async with AioClient(n_workers=2) as c:
         assert await c.submit(inc, 1) == 2
+
+
+@coro_test
+async def test_asyncio_variable():
+    async with AioClient(processes=False) as c:
+        s = c.cluster.scheduler
+
+        x = Variable('x')
+        xx = Variable('x')
+        assert x.client is c
+
+        future = c.submit(inc, 1)
+
+        await x.set(future)
+        future2 = await xx.get()
+        assert future.key == future2.key
+
+        del future, future2
+
+        await asyncio.sleep(0.1)
+        assert s.tasks  # future still present
+
+        x.delete()
+
+        start = time()
+        while s.tasks:
+            await asyncio.sleep(0.01)
+            assert time() < start + 5
